<?php
/*
Plugin Name: WP JavaScript Error Logger
Plugin URI: http://eskapism.se/wp-javascript-error-logger/
Description: View js errors
Version: 1.0
Author: Pär Thernström
Author URI: http://eskapism.se/
License: GPL2
*/

#error_reporting(E_ALL);
#ini_set('display_errors', '1');

class wp_javascript_error_logger {

	const slug = "js_error_log";

	function __construct() {

		add_action("init", array( $this, "setup") );

	}

	// Setup post type and actions on wp init
	function setup() {

		// Add custom post type that holds all our js errors
		$post_type_args = array(
			"label" => __("JS Error Log", "simple-js-error-logger"),
			"public" => true,
			"show_ui" => true,
			"rewrite" => false,
			"supports" => array("title", "custom-fields")
		);
		register_post_type(self::slug, $post_type_args);

		// Add our js to the head of the page, i.e. as early as possible
		add_action("wp_head", array( $this, "add_js_to_header" ) );

		// Add ajax action
		add_action('wp_ajax_log_js_error', array($this, "log_error") );
		add_action('wp_ajax_nopriv_log_js_error', array($this, "log_error") );

	}

	// Output some JS in header of page, that catches error
	function add_js_to_header() {
		// http://playground-nightly.ep/wordpress/
		// http://www.the-art-of-web.com/javascript/ajax-onerror/#.UXhJQWQpbQw
		?>
<script>
window.onerror = function(m, u, l) {
	// console.log('Error message: '+msg+'\nURL: '+url+'\nLine Number: '+linenumber);
	if (encodeURIComponent) {
		var ajaxurl = '<?php echo admin_url('admin-ajax.php'); ?>',
			img = new Image();
		img.src = ajaxurl + "?action=log_js_error&m=" + encodeURIComponent(m) + "&u=" + encodeURIComponent(u) + "&l=" + encodeURIComponent(l);
	}
	return true;
}
// trigger test error
runFunctionThatDoesNotExist();
</script>
<?php
	} // add js

	// Log error with ajax
	function log_error() {

		/*
		sf_d($_GET);
		Array
		(
		    [action] => log_js_error
		    [m] => Uncaught ReferenceError: runFunctionThatDoesNotExist is not defined
		    [u] => http://playground-nightly.ep/wordpress/
		    [l] => 111
		)
		*/
		// sf_d($_POST);
		$m = isset( $_GET["m"] ) ? $_GET["m"] : "";
		$u = isset( $_GET["u"] ) ? $_GET["u"] : "";
		$l = isset( $_GET["l"] ) ? $_GET["l"] : "";

		$post_arr = array(
			"post_type" => self::slug,
			"post_title" => $m
		);
		$new_post_id = wp_insert_post( $post_arr );
		
		update_post_meta( $new_post_id, "message", $m );
		update_post_meta( $new_post_id, "url", $u );
		update_post_meta( $new_post_id, "line", $l );

		exit;

	}
	
}

$GLOBALS['wp_javascript_error_logger'] = new wp_javascript_error_logger();

